#include <math.h>
/* Konstanten fuer Schraube und Kennzeichen */
   double Pi, ZPeriod, schr1, schr2 ;
   int set_schr = 1;

double rosen(double x[], int n) {
/*      See Rosenbrock (1960)		*/
   double f1, f2;
   f1 = x[1] - x[0]*x[0];
   f2 = 1. - x[0];
   return 100.*f1*f1 + f2*f2;
}
double sing(double x[], int n) {
/*      See Powell (1962)			*/
   double f1, f2, f3, f4;
   f1 = x[0] + 10.*x[1];  f2 = x[2] - x[3];
   f3 = x[1] - 2.*x[2];  f4 = x[0] - x[3];  f3 *= f3;  f4 *= f4;
   return f1*f1 + 5.*f2*f2 + f3*f3 + 10.*f4*f4;
}
double helix(double x[], int n) {
/*      See Fletcher & Powell (1963)	*/
   double r, t, f1;
   r = sqrt(x[0]*x[0] + x[1]*x[1]) - 1.;
   t = (x[0] == 0.? 0.25: atan(x[1]/x[0])/(2.*3.14159265358979));
   if (x[0] < 0.) t += .5;  f1 = x[2] - 10.*t;
   return 100.*(f1*f1 + r*r) + x[2]*x[2];
}
double cube(double x[], int n) {
/*      See Leon (1966)				*/
   double f1 = x[1] - x[0]*x[0]*x[0];
   double f2 = 1. - x[0];
   return 100.*f1*f1 + f2*f2;
}
double beale(double x[], int n) {
/*      See Beale (1958)			*/
   double f1, f2, f3;
   f1 = 1.5 - x[0] * (1. - x[1]);
   f2 = 2.25 - x[0] * (1. - x[1]*x[1]);
   f3 = 2.625 - x[0] * (1. - x[1]*x[1]*x[1]);
   return f1*f1 + f2*f2 + f3*f3;
}
double watson(double x[], int n) {
/*      See Kowalik & Osborne (1968)		*/
   double s, t, u, v;
   int i, j, k;
   u = x[0]*x[0];  v = x[1] - u - 1.;  s = u + v*v;
   k = n - 2;
   for (i=1; i<30; i++) {
	   v = (double)i/29;  t = x[n-1];
	   for (j=k; j>=0; j--) t = x[j] + v*t;
	   u = (n - 1)*x[n-1];
	   for (j=k; j>0; j--) u = j*x[j] + v*u;
	   v = u - t*t - 1.;  s += v*v;
   }
   return s;
}
double chebyquad(double x[], int n) {
/*      See Fletcher (1965)		*/
   double f, delta, tplus, d1;
   int i, j, even;
   double y[10], ti[10], tminus[10];

   delta = 0;
   for (j=0; j<n; j++) {
	   ti[j] = y[j] = 2.*x[j] - 1.;
	   delta += y[j];  tminus[j] = 1;
   }
   f = delta*delta;  even = 0;
   for (i=2; i<=n; i++) {
	   even = !even;  delta = 0;
	   for (j=0; j<n; j++) {
		   tplus = 2.*y[j]*ti[j] - tminus[j];
		   delta += tplus;  tminus[j] = ti[j];  ti[j] = tplus;
	   }
	   d1 = 1;  delta = delta/n - (even? d1/(1 - i*i): 0);
	   f += delta*delta;
   }
   return f;
}
double powell(double x[], int n) {
/*      See Powell (1964)			*/
   double f1, f2, f3;
   f1 = x[0] - x[1];  f1 = 1./(1. + f1*f1);
   f2 = sin(0.5 * 3.14159265358979 * x[1]*x[2]);
   f3 = (x[0] + x[2])/x[1] - 2.;
   return 3. - f1 - f2 - (x[1]==0.? 0.: exp(-f3*f3));
}
double wood(double x[], int n) {
/*      See Mccormick & Pearson (1969) or Colville (1968)	*/
   double f0, f1, f2, f3, f10, f32;
   f0 = x[0]-1;  f1 = x[1]-1;  f2 = x[2]-1;  f3 = x[3]-1;
   f10 = x[1] - x[0]*x[0];  f32 = x[3] - x[2]*x[2];
   return 100.*f10*f10 + f1*f1 + 90.*f32*f32 + f2*f2
	   + 10.1*(f1*f1 + f3*f3) + 19.8*f1*f3;
}
double hilbert(double x[], int n) {
/*      Computes XT.A.X, where A is the N by N Hilbert		*/
/*      matrix, see Gregory & Karney (1969), pp. 33, 66		*/
   double s = 0., t;
   int i, j;
   for (i=0; i<n; i++) {
	   t = 0;
	   for (j=0; j<n; j++) t += (double)x[j]/(i+j+1);
	   s += t*x[i];
   }
   return s;
}
double tridiag(double x[], int n) {
/*       Computes XT.A.X - 2E1T.X, whrer N > 1,

                  ( 1 -I  0  0 ... 0)
                  (-1  2 -1  0 ... 0)
                  ( 0 -1  2 -1 ... 0)
         A  =     (.................)
                  ( 0  ...  -1  2 -1)
                  ( 0  ...   0 -1  2)
         ,
         and E1T = (1, 0, ... , 0).

         See Gregory & Karney (1969), pp. 41, 74;	*/
   int i, k = n-1;
   double s = x[0] * (x[0] - x[1]);
   for (i=1; i<k; i++)
	   s += x[i] * ((x[i] - x[i-1]) + (x[i] - x[i+1]));
   s += x[k] * (2.*x[k] - x[k-1]) - 2.*x[0];
   return s;
}
double box(double x[], int n) {
/*      See Box (1966) or Brown & Dennis (1970)	*/
   int i;
   double p, s = 0., t, u;
   for (i=1; i<=10; i++) {
	   p = -(double)i/10;  t = p * x[1];
	   u = exp(p*x[0]) - ((t < -40)? 0.: exp(t))
		   - x[2] * (exp(p) - exp(10*p));
	   s += u*u;
   }
   return s;
}
double winkel(double xo, double yo) {
/* Hilfsfunktion fuer Schraube, s. unten */
  double rfi, xa, ya;
  xa = fabs(xo);  ya = fabs(yo);
  rfi = (ya < xa)? atan(ya/xa): Pi/2. - atan(xa/ya);
  if (xo < 0.) rfi = Pi - rfi;
  if (yo < 0.) rfi = -rfi;
  return rfi;
}
double schraube(double x[], int n) {
/*      Mein Schraube (helix long)		*/
   double r, t, f;
   if (set_schr) {
   /* Konstanten Pi, schr1 und schr2 vorbereiten */
	   ZPeriod = 10;
	   Pi = 1;  Pi = atan(Pi);  Pi = Pi * 4;
	   schr1 = ZPeriod / (2. * Pi);
	   schr2 = 2.*schr1*schr1;
	   set_schr = 0;
   }
   r = sqrt(x[0]*x[0] + 3.*x[1]*x[1]) - 1.;
   t = winkel(x[0],x[1]);  t = x[2]/schr1 - t;
   f = 50.*(schr2 * (1. - cos(t)) + r*r)
	   + sqrt(x[2]*x[2] + 1.) - 1.;
/* Variante:
   f = 50.*(schr2 * 4.*(1. - fabs(cos(t/2.))) + r*r)
	   + sqrt(x[2]*x[2] + 1.) - 1.;
*/
   return f;
}
